package cn.com.qmth.examcloud.core.examwork.api.controller;

import cn.com.qmth.examcloud.api.commons.enums.BooleanSelect;
import cn.com.qmth.examcloud.api.commons.enums.CourseLevel;
import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.bean.UserDataRule;
import cn.com.qmth.examcloud.api.commons.security.bean.UserType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.PathUtil;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveStudentResp;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.ExamStudentDomain;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStageRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStageEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.examwork.service.ExamStudentService;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamStudentInfo;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamRecordCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.request.CheckExamIsStartedReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.CheckExamIsStartedResp;
import cn.com.qmth.examcloud.reports.commons.bean.OperateReport;
import cn.com.qmth.examcloud.reports.commons.enums.OperateContent;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.helper.IdentityNumberHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.security.DataRule;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 考生服务API Created by songyue on 17/1/13.
 */
@RestController
@Api(tags = "考生相关接口")
@RequestMapping("${$rmp.ctr.examwork}/exam_student")
public class ExamStudentController extends ControllerSupport {

    @Autowired
    ExamStudentRepo examStudentRepo;

    @Autowired
    ExamStudentService examStudentService;

    @Autowired
    ExamRepo examRepo;

    @Autowired
    StudentCloudService studentCloudService;

    @Autowired
    CourseCloudService courseCloudService;

    @Autowired
    OrgCloudService orgCloudService;

    @Autowired
    ExamRecordCloudService examRecordCloudService;

    @Autowired
    ExamStageRepo examStageRepo;

    /**
     * 方法注释
     *
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "查询考生的专业集合")
    @GetMapping("specialtyNameList")
    public List<String> getSpecialtyNameListByStudentId() {
        User accessUser = getAccessUser();
        if (!accessUser.getUserType().equals(UserType.STUDENT)) {
            throw new StatusException("005002", "用户类型错误");
        }
        List<String> querySpecialtyNameList = examStudentRepo
                .querySpecialtyNameList(accessUser.getUserId());
        querySpecialtyNameList.removeAll(Collections.singleton(null));
        return querySpecialtyNameList;
    }

    @ApiOperation(value = "查询考生的专业集合")
    @GetMapping("courseLevelList")
    public List<String> getCourseLevelByStudentId() {
        User accessUser = getAccessUser();
        if (!accessUser.getUserType().equals(UserType.STUDENT)) {
            throw new StatusException("005001", "用户类型错误");
        }
        List<String> courseLevelList = examStudentRepo.queryCourseLevelList(accessUser.getUserId());
        List<String> ret = Lists.newArrayList();
        for (String cur : courseLevelList) {
            if (null != cur) {
                CourseLevel courseLevel = CourseLevel.getCourseLevel(cur);
                ret.add(courseLevel.getName());
            }
        }
        return ret;
    }

    /**
     * 方法注释
     *
     * @param studentId
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "查询学生的所有考试")
    @GetMapping("byStudentId/{studentId}")
    public List<ExamStudentEntity> getExamStudentListByStudentId(@PathVariable Long studentId) {

        return examStudentRepo.findByStudentId(studentId);
    }

    /**
     * 重构
     *
     * @param curPage
     * @param pageSize
     * @return
     * @author WANGWEI
     */
    @DataRule(type = {DataRuleType.ORG, DataRuleType.COURSE})
    @ApiOperation(value = "查询考试学生带条件和分页", notes = "带条件带分页")
    @GetMapping("examStudentPage/{curPage}/{pageSize}")
    public PageInfo<ExamStudentDomain> getExamStudentPage(
            @PathVariable Integer curPage,
            @PathVariable Integer pageSize,
            @RequestParam(required = false) Long orgId,
            @RequestParam(required = false) Long examId,
            @RequestParam(required = false) Long examStageId,
            @RequestParam(required = false) String studentName,
            @RequestParam(required = false) String studentCode,
            @RequestParam(required = false) Long courseId,
            @RequestParam(required = false) String courseCode,
            @RequestParam(required = false) String courseLevel,
            @RequestParam(required = false) String courseName,
            @RequestParam(required = false) String examSite,
            @RequestParam(required = false) String identityNumber,
            @RequestParam(required = false) Boolean identityNumberLike,
            @RequestParam(required = false) String specialtyName,
            @RequestParam(required = false) String infoCollector,
            @RequestParam(required = false) BooleanSelect enable,
            @RequestParam(required = false) Boolean withStarted,
            @RequestParam(required = false, defaultValue = "false") Boolean skipRule) {

        User accessUser = getAccessUser();
        PageRequest pageable = PageRequest.of(curPage, pageSize, Sort.by(Direction.DESC, "updateTime", "id"));

        UserDataRule orgDataRule = super.getUserDataRule(DataRuleType.ORG);
        UserDataRule courseDataRule = super.getUserDataRule(DataRuleType.COURSE);
        if (!skipRule) {
            if (orgDataRule.assertEmptyQueryResult() || courseDataRule.assertEmptyQueryResult()) {
                return new PageInfo(new PageImpl<>(new ArrayList<>(), pageable, 0L));
            }
        }

        Specification<ExamStudentEntity> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.equal(root.get("rootOrgId"), accessUser.getRootOrgId()));

            if (null != examId) {
                predicates.add(cb.equal(root.get("examId"), examId));
            }

            if (null != courseId) {
                predicates.add(cb.equal(root.get("courseId"), courseId));
            } else {
                if (!skipRule && courseDataRule.assertNeedQueryRefIds()) {
                    predicates.add(root.get("courseId").in(courseDataRule.getRefIds()));
                }
            }

            if (null != orgId) {
                predicates.add(cb.equal(root.get("orgId"), orgId));
            } else {
                if (!skipRule && orgDataRule.assertNeedQueryRefIds()) {
                    predicates.add(root.get("orgId").in(orgDataRule.getRefIds()));
                }
            }

            if (null != examStageId) {
                predicates.add(cb.equal(root.get("examStageId"), examStageId));
            }
            if (StringUtils.isNotEmpty(studentName)) {
                predicates.add(cb.like(root.get("name"), toSqlSearchPattern(studentName)));
            }
            if (StringUtils.isNotBlank(studentCode)) {
                predicates.add(cb.like(root.get("studentCode"), toSqlRightLike(studentCode)));
            }
            if (StringUtils.isNotEmpty(courseCode)) {
                predicates.add(cb.equal(root.get("courseCode"), courseCode));
            }
            if (StringUtils.isNotEmpty(courseLevel)) {
                predicates.add(cb.equal(root.get("courseLevel"), courseLevel));
            }
            if (StringUtils.isNotEmpty(courseName)) {
                predicates.add(cb.like(root.get("courseName"), toSqlSearchPattern(courseName)));
            }
            if (!StringUtils.isEmpty(examSite)) {
                predicates.add(cb.like(root.get("examSite"), toSqlSearchPattern(examSite)));
            }

            if (StringUtils.isNotBlank(identityNumber)) {
                final Boolean queryLike = (identityNumberLike == null) ? true : identityNumberLike;
                if (queryLike) {
                    predicates.add(cb.like(root.get("identityNumber"), toSqlRightLike(identityNumber)));
                } else {
                    predicates.add(cb.equal(root.get("identityNumber"), identityNumber));
                }
            }

            if (null != enable) {
                Boolean enableBoolean = enable.getBoolean();
                if (null != enableBoolean) {
                    predicates.add(cb.equal(root.get("enable"), enableBoolean));
                }
            }

            if (StringUtils.isNotEmpty(specialtyName)) {
                predicates.add(cb.like(root.get("specialtyName"), toSqlSearchPattern(specialtyName)));
            }

            if (StringUtils.isNotEmpty(infoCollector)) {
                predicates.add(cb.like(root.get("infoCollector"), toSqlSearchPattern(infoCollector)));
            }

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Page<ExamStudentEntity> examStudents = examStudentRepo.findAll(specification, pageable);

        List<ExamStudentDomain> ret = Lists.newArrayList();

        for (ExamStudentEntity cur : examStudents) {
            ExamEntity exam = GlobalHelper.getEntity(examRepo, cur.getExamId(), ExamEntity.class);

            GetOrgReq getOrgReq = new GetOrgReq();
            getOrgReq.setOrgId(cur.getOrgId());
            GetOrgResp getOrgResp = orgCloudService.getOrg(getOrgReq);
            OrgBean org = getOrgResp.getOrg();

            GetStudentReq getStudentReq = new GetStudentReq();
            getStudentReq.setRootOrgId(accessUser.getRootOrgId());
            getStudentReq.setIdentityNumber(cur.getIdentityNumber());
            GetStudentResp getStudentResp = studentCloudService.getStudent(getStudentReq);
            StudentBean studentBean = getStudentResp.getStudentInfo();

            Boolean started = null;
            if (null != withStarted && withStarted) {
                CheckExamIsStartedReq checkExamIsStartedReq = new CheckExamIsStartedReq();
                checkExamIsStartedReq.setExamId(exam.getId());
                checkExamIsStartedReq.setCourseId(cur.getCourseId());
                checkExamIsStartedReq.setStudentId(studentBean.getId());
                checkExamIsStartedReq.setExamId(exam.getId());
                try {
                    CheckExamIsStartedResp checkExamIsStartedResp = examRecordCloudService
                            .checkExamIsStarted(checkExamIsStartedReq);
                    started = checkExamIsStartedResp.getIsStarted();
                } catch (Exception e) {
                    LOGGER.error("fail to invoke remote method: checkExamIsStarted()", e);
                }

            }

            ExamStudentDomain bean = new ExamStudentDomain();
            bean.setRootOrgId(cur.getRootOrgId());
            bean.setId(cur.getId());
            bean.setExamId(exam.getId());
            bean.setExamName(exam.getName());
            bean.setStudentId(cur.getStudentId());
            bean.setStudentName(cur.getName());
            bean.setStudentCode(cur.getStudentCode());
            bean.setIdentityNumber(cur.getIdentityNumber());
            bean.setCourseId(cur.getCourseId());
            bean.setCourseCode(cur.getCourseCode());
            bean.setCourseName(cur.getCourseName());
            bean.setCourseLevel(cur.getCourseLevel());
            bean.setInfoCollector(cur.getInfoCollector());
            bean.setOrgId(cur.getOrgId());
            bean.setOrgCode(org.getCode());
            bean.setOrgName(org.getName());
            bean.setPaperType(cur.getPaperType());
            bean.setPhone(studentBean.getPhoneNumber());
            bean.setGrade(cur.getGrade());
            bean.setSpecialtyName(cur.getSpecialtyName());
            bean.setExamSite(cur.getExamSite());
            bean.setExamType(exam.getExamType().name());
            bean.setUpdateTime(cur.getUpdateTime());
            bean.setEnable(cur.getEnable());
            bean.setLocked(
                    null == exam.getExamStudentLocked() ? false : exam.getExamStudentLocked());
            bean.setStarted(started);
            bean.setPhotoPath(studentBean.getPhotoPath());

            bean.setExt1(cur.getExt1());
            bean.setExt2(IdentityNumberHelper.conceal(cur.getRootOrgId(), cur.getIdentityNumber()));//特殊处理：把ext2
            // 当作隐私模式下的身份证号
            bean.setExt3(cur.getExt3());
            bean.setExt4(cur.getExt4());
            bean.setExt5(cur.getExt5());
            if (null != cur.getExamStageId()) {
                ExamStageEntity examStageEntity = GlobalHelper.getEntity(examStageRepo, cur.getExamStageId(), ExamStageEntity.class);
                if (null == examStageEntity) {
                    throw new StatusException("005001", "场次id不正确");
                }

                if (examStageEntity.getEnable() == false) {
                    throw new StatusException("005002", "场次被禁用，请重新选择场次");
                }
                bean.setExamStageOrder(examStageEntity.getStageOrder());
                bean.setStartTime(examStageEntity.getStartTime());
                bean.setEndTime(examStageEntity.getEndTime());
            }
            bean.setExamStageId(cur.getExamStageId());
            ret.add(bean);
        }
        return new PageInfo<ExamStudentDomain>(examStudents, ret);
    }

    /**
     * 方法注释
     *
     * @param id
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "按ID查询考试学生", notes = "ID查询")
    @GetMapping("{id}")
    public ExamStudentEntity getExamStudentById(@PathVariable Long id) {
        ExamStudentEntity es = GlobalHelper.getEntity(examStudentRepo, id, ExamStudentEntity.class);
        if (null == es) {
            throw new StatusException("520001", "考生不存在");
        }
        return es;
    }

    /**
     * 方法注释
     *
     * @param examStudent
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "新增考试学生", notes = "新增")
    @PostMapping()
    @Transactional
    public ExamStudentDomain addExamStudent(@RequestBody ExamStudentDomain examStudent) {
        trim(examStudent);
        Long rootOrgId = getRootOrgId();
        examStudent.setRootOrgId(rootOrgId);
        examStudent.setInfoCollector(examStudent.getInfoCollector());
        ExamStudentDomain ret = saveExamStudent(examStudent);
        //操作日志
        ReportsUtil.report(new OperateReport(examStudent.getRootOrgId(), getAccessUser().getUserId(), ret.getStudentId(), ret.getId(), UserType.COMMON,
                OperateContent.EXAM_STUDENT_ADD.getDesc()));
        return ret;
    }

    /**
     * 方法注释
     *
     * @param examStudent
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "更新考试学生", notes = "更新")
    @PutMapping()
    @Transactional
    public ExamStudentDomain updateExamStudent(@RequestBody ExamStudentDomain examStudent) {
        trim(examStudent);
        Long rootOrgId = getRootOrgId();
        examStudent.setRootOrgId(rootOrgId);
        examStudent.setInfoCollector(examStudent.getInfoCollector());
        ExamStudentDomain ret = saveExamStudent(examStudent);
        //操作日志
        if (examStudent.getId() != null) {
            ReportsUtil.report(new OperateReport(examStudent.getRootOrgId(), getAccessUser().getUserId(), ret.getStudentId(), ret.getId(), UserType.COMMON,
                    OperateContent.EXAM_STUDENT_UPDATE.getDesc()));
        } else {
            ReportsUtil.report(new OperateReport(examStudent.getRootOrgId(), getAccessUser().getUserId(), ret.getStudentId(), ret.getId(), UserType.COMMON,
                    OperateContent.EXAM_STUDENT_ADD.getDesc()));
        }
        return ret;
    }

    /**
     * 方法注释
     *
     * @param examStudent
     * @return
     * @author WANGWEI
     */
    private ExamStudentDomain saveExamStudent(ExamStudentDomain examStudent) {
        SaveStudentReq saveStudentReq = new SaveStudentReq();
        saveStudentReq.setIdentityNumber(examStudent.getIdentityNumber().toUpperCase(Locale.US));
        saveStudentReq.setName(examStudent.getStudentName());
        saveStudentReq.setPhoneNumber(examStudent.getPhone());
        saveStudentReq.setOrgId(examStudent.getOrgId());
        saveStudentReq.setRemark(examStudent.getRemark());
        saveStudentReq.setRootOrgId(examStudent.getRootOrgId());
        if (StringUtils.isNotBlank(examStudent.getStudentCode())) {
            List<String> studentCodeList = Lists.newArrayList();
            studentCodeList.add(examStudent.getStudentCode());
            saveStudentReq.setStudentCodeList(studentCodeList);
        }
        SaveStudentResp saveStudentResp = studentCloudService.saveStudent(saveStudentReq);

        Long studentId = saveStudentResp.getStudentId();

        ExamStudentInfo info = new ExamStudentInfo();
        info.setCourseId(examStudent.getCourseId());
        info.setExamId(examStudent.getExamId());
        info.setIdentityNumber(examStudent.getIdentityNumber());
        info.setPaperType(examStudent.getPaperType());
        info.setRootOrgId(examStudent.getRootOrgId());
        info.setStudentCode(examStudent.getStudentCode());
        info.setStudentName(examStudent.getStudentName());
        info.setStudentId(studentId);
        info.setInfoCollector(examStudent.getInfoCollector());
        info.setGrade(examStudent.getGrade());
        info.setExamSite(examStudent.getExamSite());
        info.setSpecialtyName(examStudent.getSpecialtyName());
        info.setRemark(examStudent.getRemark());

        info.setExt1(examStudent.getExt1());
        info.setExt2(examStudent.getExt2());
        info.setExt3(examStudent.getExt3());
        info.setExt4(examStudent.getExt4());
        info.setExt5(examStudent.getExt5());

        info.setExamStageId(examStudent.getExamStageId());
        if (null != examStudent.getExamStageId()) {
            ExamStageEntity examStageEntity = GlobalHelper.getEntity(examStageRepo, examStudent.getExamStageId(), ExamStageEntity.class);
            if (null == examStageEntity) {
                throw new StatusException("005001", "场次id不正确");
            }

            if (examStageEntity.getEnable() == false) {
                throw new StatusException("005002", "场次被禁用，请重新选择场次");
            }

            info.setExamStageOrder(examStageEntity.getStageOrder());
        }

        ExamStudentInfo savedExamStudent = examStudentService.saveExamStudent(info);

        ExamStudentDomain ret = new ExamStudentDomain();
        ret.setId(savedExamStudent.getId());
        ret.setStudentId(studentId);
        ret.setStudentCode(savedExamStudent.getStudentCode());
        ret.setStudentName(savedExamStudent.getStudentName());
        ret.setCourseCode(savedExamStudent.getCourseCode());
        ret.setCourseName(savedExamStudent.getCourseName());
        return ret;
    }

    /**
     * 方法注释
     *
     * @param ids
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "按ID删除考试学生", notes = "删除")
    @DeleteMapping("/{ids}")
    @Transactional
    public List<Long> deleteExamStudent(@PathVariable String ids) {
        List<Long> idList = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        examStudentService.deleteExamStudentsByStudentIds(idList);
        return idList;
    }

    /**
     * 方法注释
     *
     * @param examId
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "按考试删除考试学生", notes = "按考试删除")
    @DeleteMapping("exam/{examId}")
    @Transactional
    public Long deleteExamStudents(@PathVariable Long examId) {
        examStudentService.deleteExamStudentsByExamId(examId);
        return examId;
    }

    @ApiOperation(value = "下载导入模板", notes = "下载导入模板")
    @GetMapping("/downloadTemplate")
    public void downloadTemplate(HttpServletResponse response) {
        String resoucePath = PathUtil.getResoucePath("templates/studentImportTemplate.xlsx");
        exportFile("考生导入模板.xlsx", new File(resoucePath));
    }

    @ApiOperation(value = "启用考生")
    @PutMapping("enable/{ids}")
    @Transactional
    public List<String> enableExamStudent(@PathVariable String ids) {
        List<Long> examStuIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        List<String> ret = Lists.newArrayList();
        for (Long cur : examStuIds) {
            ExamStudentEntity s = GlobalHelper.getEntity(examStudentRepo, cur,
                    ExamStudentEntity.class);
            s.setEnable(true);
            examStudentRepo.save(s);
            //操作日志
            ReportsUtil.report(new OperateReport(s.getRootOrgId(), getAccessUser().getUserId(), s.getStudentId(), s.getId(), UserType.COMMON,
                    OperateContent.EXAM_STUDENT_ENABLE.getDesc()));
            ret.add(s.getId() + ":" + s.getName());
        }

        for (Long cur : examStuIds) {
            examStudentService.syncExamStudent(cur);
        }

        return ret;
    }

    /**
     * 方法注释
     *
     * @param ids
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "禁用考生")
    @PutMapping("disable/{ids}")
    @Transactional
    public List<String> disableExamStudent(@PathVariable String ids) {
        List<Long> examStuIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        List<String> ret = Lists.newArrayList();
        for (Long cur : examStuIds) {
            ExamStudentEntity s = GlobalHelper.getEntity(examStudentRepo, cur,
                    ExamStudentEntity.class);
            s.setEnable(false);
            examStudentRepo.save(s);
            //操作日志
            ReportsUtil.report(new OperateReport(s.getRootOrgId(), getAccessUser().getUserId(), s.getStudentId(), s.getId(), UserType.COMMON,
                    OperateContent.EXAM_STUDENT_DISABLE.getDesc()));
            ret.add(s.getId() + ":" + s.getName());
        }

        for (Long cur : examStuIds) {
            examStudentService.syncExamStudent(cur);
        }

        return ret;
    }

}
